/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2011 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Birdstep LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file cpp-cursor.h
 *  \brief Private header for the C++ cursor API
 */

#if !defined(CURSOR_H_INCLUDED_)
#define CURSOR_H_INCLUDED_

#include "rdmtypes.h"
#include "rdmrowidtypes.h"

#if !defined(__cplusplus)
#error cpp-cursor.h is only for C++ applications.  Do not include it in C applications
#endif

namespace RDM_CPP {

    class ICursor;
    class Db;
    class Cursor;
} // namespace RDM_CPP

/*! \brief The Cursor base class
 *
 * This Cursor class provides the base cursor implementation.
 *
 * This is a class with very little overhead - a single pointer - that
 * is a 'smart pointer' (i.e. will automatically get destroyed when the last
 * reference is removed).
 *
 */
class RDM_PUBLIC_CLASS RDM_CPP::Cursor
{
  public:
    /* Handle more than one reference to the underlying cursor */
    /*! \brief Cursor Default Constructor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Cursor interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The default constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    Cursor (void) throw ();

    /*! \brief Cursor Copy Constructor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Cursor interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The copy constructor will increment the number of references to the
        underlying implementation object by one.
    */
    Cursor (const Cursor &cursor) throw ();

    /*! \brief Cursor assignment operator

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Cursor interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The assignment will increment the number of references to the underlying
        implementation object by one for the new reference.
    */
    Cursor &operator= (const Cursor &cursor) throw ();

    /*! \brief Cursor Destructor

      The default constructor, copy constructor, assignment operator, and
      destructor implement a smart pointer for the Cursor interface and it's
      generated subclasses.  The number of references to the underlying
      implementation object will be maintained.  When the number of references
      to an implementation object is down to zero the object will be released.

      The destructor will decrement the number of references to the underlying
      implementation object by one.
    */
    ~Cursor ();

    /*! \brief Get the id for the table the cursor is based on

       \return The table identifier
    */
    RDM_TABLE_ID GetTableId (void) const;

    /*! \brief Get the current info status value

        The Cursor object does not throw exceptions for status codes.  This
       function allows you to get the return code from the last method call.  If
       the previous method call did not throw and exception this will either be
       sOKAY or a status code value.  If the previous method call did throw an
       exception this will be the error code related to that exception.

       \return A RDM_RETCODE value
    */
    RDM_RETCODE GetReturnCode (void) const;

    /*! \brief Get a count of the number of rows contained in a cursor collection

        \return The number of rows contained in the cursor collection

        \par Locking Requirements
        A Snapshot transaction or a Read transaction with locks on the relevant
        cursor row(s)

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eNOTLOCKED)
    */
    uint64_t GetCount (void) const;

    /*! \brief Get a current lock status of the table a cursor is based on

        This method returns the current lock status for the table the cursor is
        based on. The lock status on specifies if the cursor currently has access
        to the table, it does not check if the table is locked by other sessions.

        \return A RDM_LOCK_STATUS value

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP)
    */
    RDM_LOCK_STATUS GetLockStatus (void) const;

    /*! \brief Mark the end of an array of table ids

        Use this when setting up an array of table ids passed in to StartRead
        or StartUpdate.

        \par Locking Requirements
        None

        \return A table id end marker
    */
    static RDM_TABLE_ID NoMoreTables (void) throw ()
    {
        return (RDM_TABLE_ID) 0;
    }

    /*! \brief Delete the row at the current cursor position

        This method removes the current row of cursor from the
        database.

        After the row has been removed the cursor will no longer have a
        current row as it has been removed from the database.  However it will
        maintain a virtual position which be the position between the
        rows neighboring the row that was deleted.  The neighboring
        rows are dependent upon on the type of the source cursor.

        \return The Cursor you are using

        \par Locking Requirements
        An Update transaction with locks on the relevant cursor row(s)

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOCURRENTROW, ::eNOSTARTUPDATE, ::eNOTLOCKED)
    */
    Cursor DeleteRow (void) const;

    /*! \brief Disconnect and Delete the row at the current cursor position

        This method disconnects the current row of cursor from all
        sets and removes it from the database.

        After the row has been removed the cursor will no longer have a
        current row as it has been removed from the database.  However it will
        maintain a virtual position which be the position between the
        rows neighboring the row that was deleted.  The neighboring
        rows are dependent upon on the type of the source cursor.

        \return The Cursor you are using

        \par Locking Requirements
        An Update transaction with locks on the relevant cursor row(s)

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOCURRENTROW, ::eNOSTARTUPDATE, ::eNOTLOCKED)
    */
    Cursor UnlinkAndDeleteRow (void) const;

    /* Basic navigation for this cursor */
    /*! \brief Navigate to the first row in the Cursor collection

        This method will position a cursor to the first row in the
        collection.  If there are no rows in the collection then the
        cursor will be positioned to \e AfterLast.

        There is a type specific version of this method in all generated subclasses.

        \return The Cursor you are using

        \par Locking Requirements
        A Snapshot transaction or a Read transaction with locks on the relevant
        cursor row(s)

        \throws rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTLOCKED)
    */
    const Cursor &MoveToFirst (void) const
    {
        return this->_MoveToFirst ();
    };

    /*! \brief Navigate to the next row in the Cursor collection

        This method will position a cursor to the next row in the
        collection.

        If the cursor was positioned to the last row in the collection then
        the cursor will be positioned to \e AfterLast.

        If cursor was positioned to \e AfterLast then the function will
        throw a rdm_exception with the ::eINVITERATION error code.

        There is a type specific version of this method in all generated subclasses.

        \return The Cursor you are using

        \par Locking Requirements
        A Snapshot transaction or a Read transaction with locks on the relevant
        cursor row(s)

        \throws rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTLOCKED)
    */
    const Cursor &MoveToNext (void) const
    {
        return this->_MoveToNext ();
    };

    /*! \brief Navigate to the previous row in the Cursor collection

        This method will position a cursor to the previous row in the
        collection.

        If the cursor was positioned to the first row in the collection
        then the cursor will be positioned to \e BeforeFirst.

        If the cursor was positioned to \e BeforeFirst then the function will
        throw rdm_exception with the ::eINVITERATION error code.

        \return The Cursor you are using

        \par Locking Requirements
        A Snapshot transaction or a Read transaction with locks on the relevant
        cursor row(s)

        \throws rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTLOCKED)
    */
    const Cursor &MoveToPrev (void) const
    {
        return this->_MoveToPrev ();
    };

    /*! \brief Navigate to the last row in the Cursor collection

        This method will position a cursor to the last row in the
        collection.  If there are no rows in the collection then the
        cursor will be positioned to \e BeforeFirst.

        There is a type specific version of this method in all generated subclasses.

        \return The Cursor you are using

        \par Locking Requirements
        A Snapshot transaction or a Read transaction with locks on the relevant
        cursor row(s)

        \throws rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTLOCKED)
    */
    const Cursor &MoveToLast (void) const
    {
        return this->_MoveToLast ();
    };

    /* Operators instead of functions above */
    /*! \brief Prefix increment operator

        This method will position a cursor to the next row in the
        collection.

        If cursor was positioned to the last row in the collection then
        the cursor will be positioned to \e AfterLast.

        If cursor was positioned to \e AfterLast then the method will
        throw a rdm_exception with the error code ::eINVITERATION.

        \return The Cursor you are using (positioned to the next row)

        \par Locking Requirements
        A Snapshot transaction or a Read transaction with locks on the relevant
        cursor row(s)

        \throws rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTLOCKED, ::eINVITERATION)
    */
    const Cursor &operator++ (void) const
    {
        return MoveToNext ();
    }

    /*! \brief Prefix decrement operator

        This method will position a cursor to the previous row in the
        collection.

        If cursor was positioned to the first row in the collection
        then the cursor will be positioned to \e BeforeFirst.

        If cursor was positioned to \e BeforeFirst then the method will
        throw a rdm_exception with the error code ::eINVITERATION.

        \return The source Cursor you are using (positioned to the previous
        row)

        \par Locking Requirements
        A Snapshot transaction or a Read transaction with locks on the relevant
        cursor row(s)

        \throws rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTLOCKED, ::eINVITERATION)
    */
    const Cursor &operator-- (void) const
    {
        return MoveToPrev ();
    }

    /*! \brief Obtain a Cursor for the current table ordered by rowId

        Create a new TableScan Cursor of rows of the same type as the current
        row of the source cursor.  Position the new cursor to the same row
        as the source cursor is positioned to.

        \return A Cursor positioned to the same row the Cursor is
                positioned to

        \par Locking Requirements
        None

        \throws rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eNOMEMORY)
    */
    Cursor GetRows (void) const;

    /*! \brief Obtain a singleton Cursor for the source cursor's current row

        Create a new singleton cursor positioned to the same row as the
        source cursor.

        If the source cursor is not at a valid row then a rdm_exception will
        be thrown.

        \return A singleton Cursor positioned at the row the source Cursor
        is positioned to

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eNOMEMORY,
       ::eOWNERDELETED, ::eSINGLETONDELETED, ::eNOCURRENTROW)
    */
    Cursor GetSelf (void) const
    {
        return this->_GetSelf ();
    };

    /*! \brief Clone a cursor

        This method crates a cursor as a clone of the source cursor.  The
        resulting cursor will be of the same type, contain the same
        rows, and will have the same position and current row as
        the source cursor.

        This is method will be intentionally hidden by the generated class
        derived for a specific database.

        \return A Cursor that is an exact clone of the source cursor

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP)
    */
    Cursor GetClone (void) const
    {
        return this->_GetClone ();
    };

    /*! \brief Obtain the static \e BeforeFirst Cursor

        The \e BeforeFirst cursor can be used in the comparison methods.
        The \e BeforeFirst cursor is positioned at the \e BeforeFirst position
       and it's position can not be changed.

        \return A singleton Cursor at the \e BeforeFirst position

        \par Locking Requirements
        None
    */
    static Cursor GetBeforeFirst (void) throw ();

    /*! \brief Obtain the static \e AfterLast Cursor

        The \e AfterLast cursor can be used in the comparison methods.
        The \e AfterLast cursor is positioned at the \e AfterLast position and
        it's position can not be changed.

        \return A singleton Cursor at the \e AfterLast position

        \par Locking Requirements
        None
    */
    static Cursor GetAfterLast (void) throw ();

    /*! \brief Check to see if the Cursor in the the \e BeforeFirst position

        \retval true If the current cursor position is before the first row in
       the collection \retval false Otherwise

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED)
    */
    bool IsBeforeFirst (void) const;

    /*! \brief Position the cursor to \e BeforeFirst

        Position the cursor to the special \e BeforeFirst position.

        Navigating to the previous position from this special position will
       position the cursor to the last row in the cursor collection.  Navigating
       forward will throw a rdm_exception with the ::eINVITERATION error code.

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED)
    */
    void MoveToBeforeFirst (void) const;

    /*! \brief Check to see if the Cursor in the the \e AfterLast position

        \retval true If the current cursor position is after the last row in the
       collection \retval false Otherwise

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED)
    */
    bool IsAfterLast (void) const;

    /*! \brief Position the cursor to \e AfterLast

        Position the cursor to the special \e AfterLast position.

        Navigating to the previous position from this special position will
       position the cursor to the last row in the cursor collection.  Navigating
       forward will throw a rdm_exception with the ::eINVITERATION error code.


        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED)
    */
    void MoveToAfterLast (void) const;

    /*! \brief Check to see where the cursor is positioned

        This function will return information about special conditions for a
       cursor's position. See \ref RDM_CURSOR_STATUS for possible cursor status
       definitions.

        As you navigate (using Next(), Prev(), First(), Last(), BeforeFirst(),
        AfterLast(), or any of the Find* functions), use InsertRow(), or create
        a cursor using any of the Db class Get methods the cursor will be in any
        of these states ::CURSOR_AT_ROW, ::CURSOR_BETWEEN, or
       ::CURSOR_BEFORE_FIRST. Changes to the row through this cursor or any
       other cursor may put the cursor in some other state. Clone will make an
       exact copy with the exact same state, position, and current row.

        \returns A RDM_CURSOR_STATUS value

        \par Locking Requirements
        None.

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    RDM_CURSOR_STATUS GetStatus (void) const;

    /*! \brief Check to see if passed in cursor is positioned before the source
        cursor's current position

        This method compares the current position of source cursor relative
        to secondary (passed in) cursor and will return TRUE if the position
        for the primary cursor is before the position of the secondary cursor.

        If the cursor types are not the same the current row of the
        secondary cursor will be used and then converted to a position in the
        cursor type of source cursor. If the current row of the secondary
        cursor is not a valid cursor position in the source a rdm_exception
        will be thrown.

        This more specifically means:

        - If the source cursor is a key cursor the comparison will be based on
       key value.
        - If the source cursor is a row cursor the comparison will be based the
       rowId
        - If the source cursor is a set member cursor the comparison will be
       based on the order in the set chain
        - If the source cursor is a singleton cursor the comparison is only
       valid if the secondary cursor and the source cursor are the same row
       (there is no order for a singleton)

          If the row at the current position of the secondary cursor is not in
       the collection of the source cursor a rdm_exception will be thrown.

          There are two distinct types of errors for the rdm_exception when the
       row in the secondary cursor is not found in the collection of the source
       cursor:

          - If rows in cursor_secondary can never be in the collection of the
       source cursor based on the schema information (it is a different table or
       from another database) then ::eNOTRELATED will be the error code in the
       rdm_exception.
          - If the row in the secondary cursor could be found in the collection
       for the source cursor but is not (it has a different set owner) then
       ::eNOTINCURSOR will be the error code in the rdm_exception.

          If the secondary cursor is in a special position (\e BeforeFirst/\e
       AfterLast) then the cursors must be of the same type to have a meaningful
       comparison (both key cursors based on the same key, both set cursors with
       the same owner, etc.). If that is not the case a rdm_exception with the
       ::eNOTINCURSOR error code is thrown.

        \retval true If the secondary cursor's position is before the source
        cursor's current position
        \retval false Otherwise

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool IsBefore (const Cursor &cursor /**< [in] The Cursor to compare against */
                   ) const;

    /*! \brief Check to see if passed in cursor is positioned before or at the
    source cursor's current position

    This method compares the current position of source cursor relative
    to secondary (passed in) cursor and will return TRUE if the position
    for the primary cursor is before or at the position of the secondary
    cursor.

    If the cursor types are not the same the current row of the
    secondary cursor will be used and then converted to a position in the
    cursor type of source cursor. If the current row of the secondary
    cursor is not a valid cursor position in the source a rdm_exception
    will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.

    \retval true If the secondary cursor's position is before or at the
    source cursor's current position
    \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool IsBeforeAt (
        const Cursor &cursor /**< [in] The Cursor to compare against */
        ) const;

    /*! \brief Check to see if passed in cursor is positioned at the same
    row as the source cursor

    This method compares the current position of source cursor relative
    to secondary (passed in) cursor and will return TRUE if the position
    for the primary cursor is at the position of the secondary cursor.

    If the cursor types are not the same the current row of the
    secondary cursor will be used and then converted to a position in the
    cursor type of source cursor. If the current row of the secondary
    cursor is not a valid cursor position in the source a rdm_exception
    will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.


    \retval true If the secondary cursor is positioned to the same row
    as the source cursor
    \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool IsAt (const Cursor &cursor /**< [in] The Cursor to compare against */
               ) const;

    /*! \brief Check to see if passed in cursor is positioned after or at the
    source cursor's current position

    This method compares the current position of source cursor relative
    to secondary (passed in) cursor and will return TRUE if the position
    for the primary cursor is at or after the position of the secondary cursor.

    If the cursor types are not the same the current row of the
    secondary cursor will be used and then converted to a position in the
    cursor type of source cursor. If the current row of the secondary
    cursor is not a valid cursor position in the source a rdm_exception
    will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.

    \retval true If the secondary cursor is positioned after or at the
    source cursor's current position
    \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool IsAfterAt (const Cursor &cursor /**< [in] The Cursor to compare against */
                    ) const;

    /*! \brief Check to see if passed in cursor is positioned after the source
                   cursor's current position.

        This method compares the current position of source cursor relative
        to secondary (passed in) cursor and will return TRUE if the position
        for the primary cursor is after the position of the secondary cursor.

        If the cursor types are not the same the current row of the
        secondary cursor will be used and then converted to a position in the
        cursor type of source cursor. If the current row of the secondary
        cursor is not a valid cursor position in the source a rdm_exception
        will be thrown.

        This more specifically means:

        - If the source cursor is a key cursor the comparison will be based on
       key value.
        - If the source cursor is a row cursor the comparison will be based the
       rowId
        - If the source cursor is a set member cursor the comparison will be
       based on the order in the set chain
        - If the source cursor is a singleton cursor the comparison is only
       valid if the secondary cursor and the source cursor are the same row
       (there is no order for a singleton)

        If the row at the current position of the secondary cursor is not in the
       collection of the source cursor a rdm_exception will be thrown.

        There are two distinct types of errors for the rdm_exception when the
       row in the secondary cursor is not found in the collection of the source
       cursor:

        - If rows in cursor_secondary can never be in the collection of the
       source cursor based on the schema information (it is a different table or
       from another database) then ::eNOTRELATED will be the error code in the
       rdm_exception.
        - If the row in the secondary cursor could be found in the collection
       for the source cursor but is not (it has a different set owner) then
       ::eNOTINCURSOR will be the error code in the rdm_exception.

        If the secondary cursor is in a special position (\e BeforeFirst/\e
       AfterLast) then the cursors must be of the same type to have a meaningful
       comparison (both key cursors based on the same key, both set cursors with
       the same owner, etc.). If that is not the case a rdm_exception with the
       ::eNOTINCURSOR error code is thrown.

        \retval true If the secondar cursor is positioned after the source
       cursor's current position \retval false Otherwise

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool IsAfter (const Cursor &cursor /**< [in] The Cursor to compare against */
                  ) const;

    /*! \brief Check to see if passed in cursor is positioned to a different
                   row than the source cursor.

        This method compares the current position of source cursor relative
        to secondary (passed in) cursor and will return TRUE if the position
        for the primary cursor is not at the position of the secondary cursor.

        If the cursor types are not the same the current row of the
        secondary cursor will be used and then converted to a position in the
        cursor type of source cursor. If the current row of the secondary
        cursor is not a valid cursor position in the source a rdm_exception
        will be thrown.

        This more specifically means:

        - If the source cursor is a key cursor the comparison will be based on
       key value.
        - If the source cursor is a row cursor the comparison will be based the
       rowId
        - If the source cursor is a set member cursor the comparison will be
       based on the order in the set chain
        - If the source cursor is a singleton cursor the comparison is only
       valid if the secondary cursor and the source cursor are the same row
       (there is no order for a singleton)

        If the row at the current position of the secondary cursor is not in the
       collection of the source cursor a rdm_exception will be thrown.

        There are two distinct types of errors for the rdm_exception when the
       row in the secondary cursor is not found in the collection of the source
       cursor:

        - If rows in cursor_secondary can never be in the collection of the
       source cursor based on the schema information (it is a different table or
       from another database) then ::eNOTRELATED will be the error code in the
       rdm_exception.
        - If the row in the secondary cursor could be found in the collection
       for the source cursor but is not (it has a different set owner) then
       ::eNOTINCURSOR will be the error code in the rdm_exception.

        If the secondary cursor is in a special position (\e BeforeFirst/\e
       AfterLast) then the cursors must be of the same type to have a meaningful
       comparison (both key cursors based on the same key, both set cursors with
       the same owner, etc.). If that is not the case a rdm_exception with the
       ::eNOTINCURSOR error code is thrown.

        \retval true If the secondary cursor is positioned to a different row
       than the source cursor. \retval false Otherwise

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool IsNotAt (const Cursor &cursor /**< [in] The Cursor to compare against */
                  ) const;

    /* Operators instead of functions above */
    /*! \brief Less than operator

        This operator compares the current position of primary (left-most)
        cursor relative to secondary (right-most) cursor and will return TRUE
        if the position for the primary cursor is before the position of the
        secondary cursor.

        If the cursor types are not the same the current row of the
        secondary cursor will be used and then converted to a position in the
        cursor type of source cursor. If the current row of the secondary
        cursor is not a valid cursor position in the source a rdm_exception
        will be thrown.

        This more specifically means:

        - If the source cursor is a key cursor the comparison will be based on
       key value.
        - If the source cursor is a row cursor the comparison will be based the
       rowId
        - If the source cursor is a set member cursor the comparison will be
       based on the order in the set chain
        - If the source cursor is a singleton cursor the comparison is only
       valid if the secondary cursor and the source cursor are the same row
       (there is no order for a singleton)

        If the row at the current position of the secondary cursor is not in the
       collection of the source cursor a rdm_exception will be thrown.

        There are two distinct types of errors for the rdm_exception when the
       row in the secondary cursor is not found in the collection of the source
       cursor:

        - If rows in cursor_secondary can never be in the collection of the
       source cursor based on the schema information (it is a different table or
       from another database) then ::eNOTRELATED will be the error code in the
       rdm_exception.
        - If the row in the secondary cursor could be found in the collection
       for the source cursor but is not (it has a different set owner) then
       ::eNOTINCURSOR will be the error code in the rdm_exception.

        If the secondary cursor is in a special position (\e BeforeFirst/\e
       AfterLast) then the cursors must be of the same type to have a meaningful
       comparison (both key cursors based on the same key, both set cursors with
       the same owner, etc.). If that is not the case a rdm_exception with the
       ::eNOTINCURSOR error code is thrown.

        \retval true If the secondary cursor's position is before the primary
       cursor's current position \retval false Otherwise

        \par Locking Requirements
        None

        \throws
        rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTRELATED, ::eNOTINCURSOR)
    */
    bool operator< (const Cursor &cursor /**< [in] The Cursor to compare against */
                    ) const
    {
        return IsBefore (cursor);
    }

    /*! \brief Less than or equal to operator

    This operator compares the current position of primary (left-most)
    cursor relative to secondary (right-most) cursor and will return TRUE
    if the position for the primary cursor is before or at the position of the
    secondary cursor.

    If the cursor types are not the same the current row of the
    secondary cursor will be used and then converted to a position in the
    cursor type of source cursor. If the current row of the secondary
    cursor is not a valid cursor position in the source a rdm_exception
    will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.

    \retval true If the secondary cursor's position is before or at the primary
    cursor's current position \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool operator<= (
        const Cursor &cursor /**< [in] The Cursor to compare against */
        ) const
    {
        return IsBeforeAt (cursor);
    }

    /*! \brief Equal to operator

    This operator compares the current position of primary (left-most)
    cursor relative to secondary (right-most) cursor and will return TRUE
    if the position for the primary cursor is at the position of the
    secondary cursor.

    If the cursor types are not the same the current row of the
    secondary cursor will be used and then converted to a position in the
    cursor type of source cursor. If the current row of the secondary
    cursor is not a valid cursor position in the source a rdm_exception
    will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.

    \retval true If the secondary cursor's position is at the primary
    cursor's current position
    \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool operator== (
        const Cursor &cursor /**< [in] The Cursor to compare against */
        ) const
    {
        return IsAt (cursor);
    }

    /*! \brief Greater than or equal to operator

      This operator compares the current position of primary (left-most)
      cursor relative to secondary (right-most) cursor and will return TRUE
      if the position for the primary cursor is after or at the position of the
      secondary cursor.

      If the cursor types are not the same the current row of the
      secondary cursor will be used and then converted to a position in the
      cursor type of source cursor. If the current row of the secondary
      cursor is not a valid cursor position in the source a rdm_exception
      will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.

    \retval true If the secondary cursor's position is after or at the
    primary cursor's current position
    \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool operator>= (
        const Cursor &cursor /**< [in] The Cursor to compare against */
        ) const
    {
        return IsAfterAt (cursor);
    }

    /*! \brief Greater than operator

        This operator compares the current position of primary (left-most)
        cursor relative to secondary (right-most) cursor and will return TRUE
        if the position for the primary cursor is after the position of the
        secondary cursor.

        If the cursor types are not the same the current row of the
        secondary cursor will be used and then converted to a position in the
        cursor type of source cursor. If the current row of the secondary
        cursor is not a valid cursor position in the source a rdm_exception
        will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.

    \retval true If the secondary cursor's position is after the primary
    cursor's current position
    \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool operator> (const Cursor &cursor /**< [in] The Cursor to compare against */
                    ) const
    {
        return IsAfter (cursor);
    }

    /*! \brief Not equal to operator

        This operator compares the current position of primary (left-most)
        cursor relative to secondary (right-most) cursor and will return TRUE
        if the position for the primary cursor is not at the position of the
        secondary cursor.

        If the cursor types are not the same the current row of the
        secondary cursor will be used and then converted to a position in the
        cursor type of source cursor. If the current row of the secondary
        cursor is not a valid cursor position in the source a rdm_exception
        will be thrown.

    This more specifically means:

    - If the source cursor is a key cursor the comparison will be based on key
    value.
    - If the source cursor is a row cursor the comparison will be based the
    rowId
    - If the source cursor is a set member cursor the comparison will be based
    on the order in the set chain
    - If the source cursor is a singleton cursor the comparison is only valid if
    the secondary cursor and the source cursor are the same row (there is no
    order for a singleton)

    If the row at the current position of the secondary cursor is not in the
    collection of the source cursor a rdm_exception will be thrown.

    There are two distinct types of errors for the rdm_exception when the row in
    the secondary cursor is not found in the collection of the source cursor:

    - If rows in cursor_secondary can never be in the collection of the source
    cursor based on the schema information (it is a different table or from
    another database) then ::eNOTRELATED will be the error code in the
    rdm_exception.
    - If the row in the secondary cursor could be found in the collection for
    the source cursor but is not (it has a different set owner) then
    ::eNOTINCURSOR will be the error code in the rdm_exception.

    If the secondary cursor is in a special position (\e BeforeFirst/\e
    AfterLast) then the cursors must be of the same type to have a meaningful
    comparison (both key cursors based on the same key, both set cursors with
    the same owner, etc.). If that is not the case a rdm_exception with the
    ::eNOTINCURSOR error code is thrown.

    \retval true If the secondary cursor's position is not the same as the
    primary cursor's current position
    \retval false Otherwise

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOTRELATED)
    */
    bool operator!= (
        const Cursor &cursor /**< [in] The Cursor to compare against */
        ) const
    {
        return IsNotAt (cursor);
    }

    /*! \brief Get the Db object associated with the Cursor

    Each active cursor is associated with a Db object.  This method allows
    a user to get a reference to the Db object associated with the source
    Cursor.

    \return The Db object associated with this Cursor

    \par Locking Requirements
    None

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP)
    */
    Db GetDb (void);

    /*! \brief Get the rowid of Cursor's current row

    This method returns the database address of the current row for
    the Cursor.

    If the cursor is not at a valid row a rdm_exception will be thrown.

    \return The database address of the row

    \par Locking Requirements
    A Snapshot transaction or a Read transaction with locks on the relevant
    cursor row(s)

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOCURRENTROW, ::eNOMEMORY, ::eNOTLOCKED)
    */
    RDM_ROWID_T GetRowId () const;

    /*! \brief Get the row status information of a status cursor

    This retrieves returns the ::RDM_ROW_STATUS_INFO for a status cursor.  A
    status cursor can be retrieved from a bulk insert operation.

    If the cursor is not at a valid entry a rdm_exception will be thrown.

    \par Locking Requirements
    None

    \throws
    rdm_exception (::eINVCURSOROP, ::eNOCURRENTROW)
    */
    void GetRowStatus (RDM_ROW_STATUS_INFO &rowStat) const;

    /*! \brief Position a cursor to a particular row by database address

       This method will position a cursor to the row at database
       address specified by dba.  The cursor can be any type of cursor and
       does not need to be a row scan cursor.

       If there is no exact match found then:

       - If the cursor is a row scan cursor and dba is in the file
       that row is stored in then the function will return
       ::sNOTFOUND and the cursor will be placed in the
       ::CURSOR_BETWEEN position.  Navigating forward (next)
       from that position will position the cursor at the next DBA
       larger than the specified DBA.  Navigating backward (prev)
       from that position will position the cursor at the next DBA
       smaller than the specified DBA.

       - Otherwise the function will throw a rdm_exception with the
       ::eNOTINCURSOR error code and the cursor position will be left
       unchanged.

       \par Locking Requirements
       A Snapshot transaction or a Read transaction with locks on the relevant
       cursor row(s)

       \throws
       rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
       ::eSINGLETONDELETED, ::eNOTRELATED)
       */
    void MoveToRowId (
        const RDM_ROWID_T
            rowId /**< [in] The id of the row the Cursor is to be positioned to*/
        ) const;

    /*! \brief Add a new member to a set cursor

    This method will add a new member row to a set cursor.  The passed
    in cursor must be positioned to a row that is a valid member of
    the set the source cursor is based on.  The current row of the passed
    in cursor will be added as a member of the set the source cursor is
    based on and will be associated to the current owner row of the
    source cursor.

    An exception will be thrown if the source cursor is not a set cursor
    or if the member cursor is not positioned to a valid member row.

    \par Locking Requirements
    An Update transaction with locks on the relevant cursor row(s)

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED,
    ::eSINGLETONDELETED, ::eNOCURRENTROW, ::eNOTLOCKED, ::eNOSTARTUPDATE)
    */
    void AddMember (
        const Cursor &member /**< A cursor positioned to a row that is a valid
                                member for the set the source Cursor is based on */
        ) const;

    /*! \brief Remove the current row from a set cursor

    This function disconnects the current row of the source cursor from
    the set associated with the cursor.  The source cursor must be based on a set
    (obtained from a call to a GetMember* method).

    The source cursor will maintain the original current row and position
    even though the current row is no longer in the set.  Calling
    next/prev will position the cursor relative to the original position.

    \par Locking Requirements
    An Update transaction with locks on the relevant cursor row(s)

    \throws
    rdm_exception (::ePRECOMMITTED, ::eINVCURSOROP, ::eOWNERDELETED, ::eSINGLETONDELETED,
    ::eNOCURRENTROW, ::eNOTLOCKED, ::eNOSTARTUPDATE, ::eNOTLINKED)
    */
    void RemoveMember (void) const;

    /*! \internal

        \brief Release the cursor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the Cursor interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    Cursor &Release (void);

#ifndef RDM_DOXYGEN
  protected:
    ICursor *m_icursor;

    const Cursor &_MoveToFirst (void) const;
    const Cursor &_MoveToNext (void) const;
    const Cursor &_MoveToLast (void) const;
    const Cursor &_MoveToPrev (void) const;

    Cursor _GetSelf (void) const;
    Cursor _GetClone (void) const;

    void VerifyTableId (RDM_TABLE_ID) const;

    Cursor &_AssignAndVerifyTableId (RDM_TABLE_ID, const Cursor &);
    Cursor (ICursor *) throw ();
    Cursor (const Cursor &, RDM_TABLE_ID);

    Cursor _GetOwnerRow (RDM_REF_ID) const;
    RDM_ROWID_T _GetOwnerRowId (RDM_REF_ID) const;

    Cursor _GetMemberRows (RDM_REF_ID) const;
    Cursor _GetSiblingRows (RDM_REF_ID, bool) const;
    Cursor _GetRowsByKeyAtPosition (RDM_KEY_ID keyId) const;

    bool _HasOwner (RDM_REF_ID refId) const;
    uint64_t _GetMemberCount (RDM_REF_ID) const;
    void _Connect (RDM_REF_ID, const Cursor &, bool) const;
    void UnlinkRow (RDM_REF_ID) const;

    void _MoveToKey (RDM_KEY_ID, const void *, size_t = 0) const;
    void _MoveToKey (RDM_KEY_ID, const RDM_SEARCH_KEY *) const;

    void _UpdateRow (const void *, size_t = 0) const;
    void _ReadRow (void *, size_t = 0, size_t * = NULL) const;
    void _UpdateColumn (RDM_COLUMN_ID, const void *, size_t = 0) const;
    void _ReadColumn (RDM_COLUMN_ID, void *, size_t = 0, size_t * = NULL) const;
    void _UpdateBlob (RDM_COLUMN_ID, uint64_t, const void *, size_t) const;
    void _ReadBlob (RDM_COLUMN_ID, uint64_t, void *, size_t, size_t * = NULL) const;
    void _SetBlobSize (RDM_COLUMN_ID, uint64_t) const;
    void _GetBlobSize (RDM_COLUMN_ID columnId, uint64_t *) const;
    void _ValidateICursor () const;

  private:
    const Cursor *operator& (void) const;
#endif
    friend class Db;
};
#endif /* ! CURSOR_H_INCLUDED_ */
