/* ----------------------------------------------------------------------------
 *
 * Raima Database Manager
 *
 * Copyright (c) 2011 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Birdstep LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 *
 * ----------------------------------------------------------------------------
 */

/** \file cpp-tfs.h
 *  \brief Header for the C++ Transaction File Server (TFS) API
 */

#if !defined(CPP_TFS_H)
#define CPP_TFS_H

#if !defined(__cplusplus)
#error cpp-tfs.h is only for C++ applications.  Do not include it in C applications
#endif

#include "cpp-exception.h"

#if defined(RDM_WINDOWS)
#pragma warning(disable : 4290)
#endif

#include "rdmtfstypes.h"

/** \defgroup rdm_cpp RDM C++ APIs
 *
 * The RDM C+ APIs. You may want to start by instantiation a TFS using
 * RDM_CPP::TFS::Alloc(), then   Please see the respective sub-modules for details.
 */

/** \brief The RDM C++ Namespace
 *
 *  \ingroup rdm_cpp
 */
namespace RDM_CPP {
    class Cursor;
    class Db;
    class ITFS;
    class TFS;
} // namespace RDM_CPP

#include "psptypes.h"

/*! \brief The TFS class
   This class provides the TFS implementation

   This is a class with very little overhead - a single pointer - that
   is a 'smart pointer' (i.e. will automatically get destroyed when the last
   reference is removed).

   To keep the overhead down this class should have no virtual functions.
  */
class RDM_PUBLIC_CLASS RDM_CPP::TFS
{
  public:
    /* Handle more than one reference to the underlying database */

    /*! \brief TFS Default Constructor

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the TFS interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The default constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    TFS (void) throw ();

    /*! \brief TFS assignment operator

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the TFS interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The assignment operator will increment the number of references to the
        underlying implementation object by one.
    */
    TFS &operator= (const TFS &tfs) throw ();

    //! Validate the ITFS
    // void _ValidateITFS () const;
    void _ValidateITFS () const
    {
        if (m_itfs == NULL)
        {
            throw (rdm_exception (eINVTFSID));
        }
    }

    /** \brief Return TFS object using the specified RDM_TFS

        This method will allocate the TFS service and return its handle.

        \par Locking Requirements
        None
    */
    static TFS Alloc (RDM_TFS tfs /**< [in] RDM_TFS handle */
    );

    /** \brief Allocate an RDM_TFS and return a TFS object

        This method will allocate the TFS service with specified
        option values and return its handle. The \a optString is a key/value
        pair in the form of "key=value".

        \includedoc dox/tfs-options.mdinc

        \note Database configuration options can be set using this function and
       will be inherited by the database handle at the time it is allocated. For
        example, the \c{"durability=consistent"} option can be set using this
        function and it will be inherited by the next Db handle allocated
        using this TFS handle. See \ref sqldb_options for the list of valid
        database configuration option keys.
    */
    static TFS Alloc (const RDM_CHAR_T *options, /**< [in] TFS option string */
                      size_t memSize = 0, /**< [IN] The memory size limit (size of memBuff if memBuf is specified) */
                      void *memBuf = NULL /**< [IN] The memory buffer to use instead of getting memory from the operating system */
    );

#if defined(RDM_UNICODE)
    static TFS Alloc (
        const wchar_t *options,
        size_t memSize = 0,
        void *memBuf = NULL
    );
#endif

    /** \brief Instantiate a database
     *
     *  This method will allocate the DB with specified options and
     *  return its handle. The \a optString is a key/value pair in the
     *  form of "key=value".
     *
     *  \returns A database
     */
    Db AllocDatabase (
        const RDM_CHAR_T *opts = NULL /**< [in] DB option string */
    );

#if defined(RDM_UNICODE)
    Db AllocDatabase (const wchar_t *opts);
#endif

    /** \brief Set TFS option values

    This method is used to set any RDM TFS options. The \a optString is a
    key/value pair in the form of "key=value". The options are inherited by
    the database and/or SQL handle derived from this TFS handle.

    \includedoc dox/sqldb-options.mdinc

    \throws
    rdm_exception
    */
    void SetOptions (
        const RDM_CHAR_T *optString /**< [in] A null terminated
                                       string containing the key/value pairs */
    );
#if defined(RDM_UNICODE)
    void SetOptions (const wchar_t *optString);
#endif

    /** \brief Set a TFS option value

    This method sets an option for the TFS Handle

    \par Locking Requirements
    None

    \throws
    rdm_exception
    */
    void SetOption (
        const RDM_CHAR_T
            *key, /**< [in] A null terminated name of the option to be set. */
        const RDM_CHAR_T
            *value /**< [in] A null terminated value for the specified option */
    );

#if defined(RDM_UNICODE)
    void SetOption (const wchar_t *key, const wchar_t *value);
#endif

    /** \brief \copybrief rdm_tfsGetOptions()

    \details The method retrieves a semicolon-delimited list of options in the
    pair of keyword and value for the options specified in 'keywords.'

    \a keywords should be a list of option values. If \a keywords is NULL, the
    function returns the pairs for all available options. If \a keywords is an
    empty string, the function returns an empty string.

    \throws
    rdm_exception
    */
    void GetOptions (
        RDM_CHAR_T *optionString, /**< [out] Option string */
        size_t bytesIn, /**< [in] the size of optionString (in bytes) */
        size_t *bytesOut =
            NULL /**< [out] the number of bytes written to optionString */
    );
#if defined(RDM_UNICODE)
    void GetOptions (wchar_t *optionString, size_t bytesIn, size_t *bytesOut = NULL);
#endif
    /** \brief Get a TFS option value

    This method retrieves an option value for the TFS Handle

    \par Locking Requirements
    None

    \throws
    rdm_exception
    */
    void GetOption (
        const RDM_CHAR_T
            *key, /**< [in] A null terminated name of the option to be set. */
        RDM_CHAR_T *value, /**< [in] A null terminated value for the specified option */
        size_t bytesIn,         /**< [in] the size of value (in bytes) */
        size_t *bytesOut = NULL /**< [out] the number of bytes written to value */
    );

#if defined(RDM_UNICODE)
    void GetOption (
        const wchar_t *key,
        wchar_t *value,
        size_t bytesIn,
        size_t *bytesOut = NULL);
#endif

    /** \copybrief rdm_tfsDisableListener()
     */
    void DisableListener (void) const;

    /** \copybrief rdm_tfsEnableListener()
     */
    void EnableListener () const;

    /** \brief Set an encryption context
     */
    void GetEncrypt (RDM_ENCRYPT *enc) const;

    /** \brief Get the list of users connected to the TFS

        This method returns an array of user IDs, each of which represents
        a TFS connection.
    */
    void GetConnections (
        const RDM_CHAR_T *uri,                 /**< [in]  Connection URI */
        const RDM_CHAR_T *const **ppszUserIDs, /**< [out] Array of user IDs */
        uint32_t *puUserCount                  /**< [out] Number of user IDs */
        ) const;

#if defined(RDM_UNICODE)
    void GetConnections (
        const wchar_t *uri,
        const wchar_t *const **ppszUserIDs,
        uint32_t *puUserCount) const;
#endif

    /** \brief Get the TFS type and version

        This method will obtain the type and version of the TFS
    */
    void GetVersion (
        uint16_t *pMajorV, /**< [out] major version */
        uint16_t *pMinorV, /**< [out] minor version */
        TFS_TYPE *pTfsType /**< [out] TFS type */
        ) const;

    RDM_ENCRYPT AllocEncrypt (const char *passcode);

#if defined(RDM_UNICODE)
    RDM_ENCRYPT AllocEncrypt (const wchar_t *passcode);
#endif

    /** \brief Get the RDM_TFS handcle for a TFS object

        \return The RDM_TFS handle for the TFS object
    */
    RDM_TFS GetTFS () const;

    /** \brief Get the RDM_TFS type for a TFS object

        \return The TFS_TYPE for the TFS object
    */
    TFS_TYPE GetType (void) const;

    /** \brief Kill the TFS connection associated with a user
     */
    void KillRemoteConnection (
        const RDM_CHAR_T *uri,     /**< [in]  Connection URI for TFSR */
        const RDM_CHAR_T *dbUserID /**< [in]  User whose connection to kill */
        ) const;

#if defined(RDM_UNICODE)
    void KillRemoteConnection (const wchar_t *uri, const wchar_t *dbUserID) const;
#endif

    /** \brief Kill all the TFS connections
     */
    void KillAllRemoteConnections (
        const RDM_CHAR_T *uri /**< [in]  Connection URI for TFSR */
        ) const;

#if defined(RDM_UNICODE)
    void KillAllRemoteConnections (const wchar_t *uri) const;
#endif

    /** \brief Ping a remote TFS

        This method will ping the TFS specified by the URI to see if the TFS
        is active.
    */
    void Ping (const RDM_CHAR_T *uri /**< [in]  Connection URI for TFSR */
               ) const;

#if defined(RDM_UNICODE)
    void Ping (const wchar_t *uri) const;
#endif

    /** \brief Drop a database

        This method will drop the specified database.
    */
    void DropDatabase (
        const RDM_CHAR_T *dbNameSpec /**< [in]  The database name specification
                                        for the database to be dropped*/
        ) const;

    /** \brief Get memory usage information

        This method will get the TFS memory usage information
    */
    void GetMemoryUsage (
        size_t &currUsage, /**< [out] The current amount of memory in use by the TFS */
        size_t &maxUsage /**< [out] The current amount of memory in use by the TFS */
    ) const;

#if defined(RDM_UNICODE)
    void DropDatabase (const wchar_t *dbNameSpec) const;
#endif

    /** \brief Get a capability

        This method get an information about whether a capability is supported by a TFS
    */
    RDM_BOOL_T GetCapability (
        const RDM_CHAR_T *name /**< [in]  The name of the capability to query for */
        ) const;

#if defined(RDM_UNICODE)
    RDM_BOOL_T GetCapability (const wchar_t *name) const;
#endif

    /*! \internal

        \brief Release the TFS

        The default constructor, copy constructor, assignment operator, and
        destructor implement a smart pointer for the TFS interface and it's
        generated subclasses.  The number of references to the underlying
        implementation object will be maintained.  When the number of references
        to an implementation object is down to zero the object will be released.

        The constructor will initialize the number of references to the
        underlying implementation object to one.
    */
    TFS &Release (void);

    /*! \brief TFS destructor

       The default constructor, copy constructor, assignment operator, and
       destructor implement a smart pointer for the TFS interface and it's
       generated subclasses.  The number of references to the underlying
       implementation object will be maintained.  When the number of references
       to an implementation object is down to zero the object will be released.

       The destructor will decrement the number of references to the underlying
       implementation object by one.
   */
    ~TFS ();
    /** \brief Constructor

      Applications should not directly create TFS classes.  They should use
      the static Alloc method to get a TFS class.
    */
    TFS (ITFS *itfs /**< [in] The internal ITFS to use with this instance */
    )
    throw ();

    /** \brief Constructor

      Applications should not directly create TFS classes.  They should use
      the static Alloc method to get a TFS class.
     */
    TFS (const TFS &tfs /**< [in] an existing TFS to base this instance on */
    )
    throw ();

    /*!

    The TFS object does not throw exceptions for status codes.  This function
    allows you to get the return code from the last method call.  If the
   previous method call did not throw and exception this will either be sOKAY or
   a status code value.  If the previous method call did throw an exception this
   will be the error code related to that exception.

   \return The current RDM_RETCODE value
    */
    RDM_RETCODE GetReturnCode () const;

  protected:
    //! Internal ITFS pointer
    ITFS *m_itfs;

  private:
    const TFS *operator& (void) const;

    friend class Db;
};

#endif
