/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2010 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

/** \file rdmtimestampapi.h
 *  \brief Header for the public timestamp API
 */

#include "rdmdatetimetypes.h"
#include "rdmretcodetypes.h"

#if defined(RDM_UNICODE)
#define rdm_timestampDayAbr RDM_T_FUNC (rdm_timestampDayAbr)
#define rdm_timestampDayName RDM_T_FUNC (rdm_timestampDayName)
#define rdm_timestampFromString RDM_T_FUNC (rdm_timestampFromString)
#define rdm_timestampMonthAbr RDM_T_FUNC (rdm_timestampMonthAbr)
#define rdm_timestampMonthName RDM_T_FUNC (rdm_timestampMonthName)
#define rdm_timestampNowAsString RDM_T_FUNC (rdm_timestampNowAsString)
#define rdm_timestampToString RDM_T_FUNC (rdm_timestampToString)
#endif

/** \defgroup timestamp Timestamp API
 * \ingroup datetime
 *
 * The RDM timestamp datatype API.  The functions here are located in \lib{base}
 *
 * \addtogroup timestamp
 * @{
 */

#if defined(__cplusplus)
extern "C" {
#endif

/** \brief Get the day abbreviation
 *
 *  Get the three letter day abbreviation in English from a packed local timestamp.
 *
 * \return The day abbreviation
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_timestampDayAbr (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the day name
 *
 *  Get the day name in English from a packed local timestamp.
 *
 * \return The day name
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_timestampDayName (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the day of the month
 *
 *  Get the day of month from a packed local timestamp.
 *
 * \return The day of month (1 - 31)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampDayOfMonth (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the day of the week
 *
 *  Get the day of week from a packed local timestamp for a US locale where
 *  Sunday is the first day of the week.
 *
 * \return The day of week (1 - 7)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampDayOfWeek (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the day of the year
 *
 *  Get the day of year from a packed local timestamp where January 1st is the
 *  first day.
 *
 * \return The day of the year (1 - 366)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampDayOfYear (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the difference between two timestamps
 *
 *  Get the difference between two timestamps measured in the number of whole days.
 */
RDM_EXPORT void EXTERNAL_FCN rdm_timestampDaysDiff (
    RDM_PACKED_TIMESTAMP_T start, /**< [in]  start timestamp */
    RDM_PACKED_TIMESTAMP_T end,   /**< [in]  end timestamp */
    uint32_t *diff /**< [out] The difference in whole number of days */
);

/** \brief Convert from a string to a timestamp
 *
 *  Convert a string to a packed local timestamp.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVDATE}
 *  \retval{eINVDATEFMT}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestampFromString (
    const RDM_TCHAR_T *str,     /**< [in]  timestamp value in string */
    RDM_DATE_FORMAT date_fmt,   /**< [in]  RDM date format */
    RDM_PACKED_TIMESTAMP_T *ptv /**< [out] The packed local timestamp */
);

/** \brief Get the hour of the day
 *
 *  Get the hour from a packed local timestamp.
 *
 *  \return The hour of the day
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampHour (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the minute of the hour
 *
 *  Get the minute of the hour from a packed local timestamp.
 *
 *  \return The minute of the hour
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampMinute (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the second of the minute
 *
 *  Get the second of the minute from a packed local timestamp.
 *
 *  \return The second of the minute
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampSecond (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the fractions of the second
 *
 *  Get the fractions of the second in 1/10,000th of a second from a packed local time.
 *
 *  The fractions of the second (0 - 9999)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampFraction (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the day of the month
 *
 *  Get the day of month from a packed local timestamp.
 *
 * \return The day of month (1 - 31)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampMonth (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the month abbreviation
 *
 * Get the three letter month abbreviation in English
 *
 * \return The month abbreviation
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_timestampMonthAbr (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the month name
 *
 * Get the month name in English from a packed local timestamp.
 *
 * \return The month name
 */
RDM_EXPORT const RDM_TCHAR_T *EXTERNAL_FCN rdm_timestampMonthName (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the current local timestamp
 *
 *  Get the current local timestamp for a timezone.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVTIMEZONE}
 *
 *  \see rdm_timestampNowAsString
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestampNow (
    int16_t time_zone,         /**< [in]  Timezone value (in min) */
    RDM_PACKED_TIMESTAMP_T *ts /**< [out] local timestamp value */
);

/** \brief Get the current local timestamp as string
 *
 *  Get the current local timestamp as a string.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVTIMEZONE}
 *  \retval{sTRUNCATE}
 *
 *  \see rdm_dateToday
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestampNowAsString (
    int16_t time_zone,  /**< [in]  Timezone value (in min) */
    RDM_TCHAR_T *tsbuf, /**< [out] NOW value in string */
    size_t buflen /**< [in]  Size of tsbuf in number of ASCII characters it can hold */
);

/** \brief Get the current timestamp as a double
 *
 *  Get the current timestamp as a double.
 *
 *  \return The current timestamp
 */
RDM_EXPORT double EXTERNAL_FCN rdm_timestampNowAsDouble (void);

/** \brief Get the quarter of the year
 *
 *  Get the quarter of the year from a packed local timestamp.
 *
 *  \return The quarter of the year (1 - 4)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampQuarter (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Convert a timestamp to a string
 *
 *  Convert a packed local timestamp to a string.
 *
 *  \retval{sOKAY}
 *  \retval{eBASE_INVDATE}
 *  \retval{eINVDATESEP}
 *  \retval{sTRUNCATE}
 */
RDM_EXPORT RDM_RETCODE EXTERNAL_FCN rdm_timestampToString (
    RDM_PACKED_TIMESTAMP_T tmsv, /**< [in]  RDM timestamp value */
    RDM_DATE_FORMAT date_fmt,    /**< [in]  RDM date format */
    RDM_TCHAR_T date_sep,        /**< [in]  date separator */
    RDM_TIME_FORMAT time_fmt,    /**< [in]  RDM time format */
    RDM_TCHAR_T *buf,            /**< [out] result timestamp string */
    size_t bufSize, /**< [in]  buffer size in number of ASCII characters it can hold */
    size_t *puSize /**< [out] resulting size in number of characters */
);

/** \brief Get the week of the year
 *
 *  Get the week of the year in US locale.
 *
 *  \return The week of the year (1 - 53)
 */
RDM_EXPORT uint16_t EXTERNAL_FCN rdm_timestampWeek (
    RDM_PACKED_TIMESTAMP_T ts /**< [in]  RDM timestamp value */
);

/** \brief Get the year
 *
 *  Get the year from a packed local timestamp.
 *
 *  \return The yead
 */
RDM_EXPORT int32_t EXTERNAL_FCN rdm_timestampYear (RDM_PACKED_TIMESTAMP_T ts);

/** \brief Get the timestamp of the epoch
 *
 *  Get the local timestamp of the epoch.
 *
 *  \return The timestamp of the epoch
 */
RDM_EXPORT RDM_PACKED_TIMESTAMP_T EXTERNAL_FCN rdm_timestampZero (void);

#if defined(__cplusplus)
}
#endif

/** @}*/
